import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  transition,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let xLabel;
  let yLabel;
  let margin;
  let radius;

  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    const t = transition().duration(1000);

    const positionCircles = (circles) => {
      circles.attr('cx', (d) => d.x).attr('cy', (d) => d.y);
    };

    const initializeRadius = (circles) => {
      circles.attr('r', 0);
    };
    const growRadius = (enter) => {
      enter.transition(t).attr('r', radius);
    };

    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join(
        (enter) =>
          enter
            .append('circle')
            .call(positionCircles)
            .call(initializeRadius)
            .call(growRadius),
        (update) =>
          update.call((update) =>
            update
              .transition(t)
              .delay((d, i) => i * 10)
              .call(positionCircles)
          ),
        (exit) => exit.remove()
      );

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .transition(t)
      .call(axisLeft(y));

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .transition(t)
      .call(axisBottom(x));


    // Labels
    const text = selection
      .selectAll('g.x-label')
      .data([null])
      .join('g')
      .attr('class', 'x-label')
      .selectAll('text')
      .data(xLabel, (d, i) => `${d}_${i}`)
      .join(
        (enter) =>
          enter
            .append('text')
            .attr(
              'transform',
              `translate( ${
                (width - margin.right + margin.left) / 2.8
              }, ${height - margin.bottom / 2})`
            )
            .attr('fill', 'green')
            .attr('y', -7)
            .attr('dy', '0.35em')
            .attr('x', (d, i) => i * 30)
            .text((d) => d),
        (update) => update.attr('fill', 'black'),
        (exit) =>
          exit
            .attr('fill', 'brown')
            .call((text) =>
              text.transition(t).remove().attr('y', 50)
            )
      )
      .call((text) =>
        text
          .transition(t)
          .attr('y', 17)
          .attr('x', (d, i) => i * 30)
      );

    selection
      .selectAll('.y-label')
      .data([null])
      .join('text')
      .text(yLabel)
      .attr('class', 'y-label')
      .attr('text-anchor', 'middle')
      .attr(
        'transform',
        `translate(${margin.left / 3},${
          (height - margin.bottom + margin.top) / 2
        }) rotate(-90)`
      );
  };

  // accessors

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.xLabel = function (_) {
    return arguments.length ? ((xLabel = _), my) : xLabel;
  };

  my.yLabel = function (_) {
    return arguments.length ? ((yLabel = _), my) : yLabel;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
